// 
// Copyright (C) 2008, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
    /// </summary>
    [Description("Reference sample demonstrating how to use DataSeries objects to store self calculations.")]
    public class SampleCustomDataSeries : Indicator
    {
        #region Variables
		private int period = 5;
		
		// Defines the DataSeries object
		private DataSeries myDataSeries;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			// Adds a plot to our NinjaScript Indicator
            Add(new Plot(Color.Orange, PlotStyle.Bar, "Average Range"));
			
			// Create a new DataSeries object and assign it to the variable myDataSeries declared in the ‘Variables’ region above
			myDataSeries = new DataSeries(this);
			
            CalculateOnBarClose	= true;
            Overlay				= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			/* To set values to our DataSeries we use the Set() method. Here we are setting the DataSeries
			object for the current bar to take on the absolute value of the difference between the current bar's
			open and close. */
			myDataSeries.Set(Math.Abs(Close[0] - Open[0]));
			
			/* Take note that the method for setting the value to be plotted is the same as for setting a value
			to a DataSeries object. The difference here is that the custom DataSeries object is not plotted while
			this "AvgR" is plotted.
			
			In this case we are plotting the Simple Moving Average of the intermediate calculation step stored in our
			DataSeries object. */
            AvgR.Set(SMA(myDataSeries, Period)[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries AvgR
        {
            get { return Values[0]; }
        }

        [Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleCustomDataSeries[] cacheSampleCustomDataSeries = null;

        private static SampleCustomDataSeries checkSampleCustomDataSeries = new SampleCustomDataSeries();

        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        public SampleCustomDataSeries SampleCustomDataSeries(int period)
        {
            return SampleCustomDataSeries(Input, period);
        }

        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        public SampleCustomDataSeries SampleCustomDataSeries(Data.IDataSeries input, int period)
        {
            checkSampleCustomDataSeries.Period = period;
            period = checkSampleCustomDataSeries.Period;

            if (cacheSampleCustomDataSeries != null)
                for (int idx = 0; idx < cacheSampleCustomDataSeries.Length; idx++)
                    if (cacheSampleCustomDataSeries[idx].Period == period && cacheSampleCustomDataSeries[idx].EqualsInput(input))
                        return cacheSampleCustomDataSeries[idx];

            SampleCustomDataSeries indicator = new SampleCustomDataSeries();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.SetUp();

            SampleCustomDataSeries[] tmp = new SampleCustomDataSeries[cacheSampleCustomDataSeries == null ? 1 : cacheSampleCustomDataSeries.Length + 1];
            if (cacheSampleCustomDataSeries != null)
                cacheSampleCustomDataSeries.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSampleCustomDataSeries = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleCustomDataSeries SampleCustomDataSeries(int period)
        {
            return _indicator.SampleCustomDataSeries(Input, period);
        }

        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleCustomDataSeries SampleCustomDataSeries(Data.IDataSeries input, int period)
        {
            return _indicator.SampleCustomDataSeries(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleCustomDataSeries SampleCustomDataSeries(int period)
        {
            return _indicator.SampleCustomDataSeries(Input, period);
        }

        /// <summary>
        /// Reference sample demonstrating how to use DataSeries objects to store self calculations.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleCustomDataSeries SampleCustomDataSeries(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleCustomDataSeries(input, period);
        }

    }
}
#endregion
